# -*- coding: utf-8 -*-
# cfgmng.py

"""
Gestion fichiers de configurations (YAML)

pathfile : dreamtools/cfgmng.py

Repertoires par défaut
----------------------
.. note::
    * PROJECT_DIR/cfg/PROJECT_DIR/cfg/.log.yml : Fichier de configuration des logs
    * PROJECT_DIR/cfg/.app.yml : Fichier de configuration de l'application
    * PROJECT_DIR/cfg/categorie.yml : Fichier de liste définie par un code et un libelle
    * PROJECT_DIR/cfg/mailing.yml : Fichier de mails préparés
    * PROJECT_DIR/cfg/validators.yml : Fichier de validation(cf CERBERUS)
    * PROJECT_DIR/cfg/normalizor.yml : Fichier de normalization(cf CERBERUS)

Class CFBases
-------------
"""

import yaml

try:
    from yaml import CLoader as Loader, CDumper as Dumper, CSafeLoader as SafeLoader
except ImportError:
    from yaml import Loader, Dumper, SafeLoader

from . import tools
from . import profiler


class CFGEngine(object):
    """
    cfg engine
    """
    __dirpath = None

    @staticmethod
    def initial(baz_dir='cfg'):
        """

        :param str baz_dir:
        :return:
        """
        CFGEngine.__dirpath = profiler.path_build(tools.PROJECT_DIR, baz_dir)

    @staticmethod
    def working_directory(sub_dir):
        CFGEngine.initial()
        return profiler.path_build(CFGEngine.__dirpath, sub_dir)

    @staticmethod
    def loading(p, ref=None, m='r'):
        """
        Récupération des parametres de configuration du fichier <p> section <r>

        :param str p: Fichier de configuration
        :param str ref: référence parametres à récupérer, optionnel
        :param str m: bytes par defaut
        :return: configuration | None

        """
        config = None

        try:
            if profiler.file_exists(p):
                with open(p, mode=m) if 'b' in m else open(p, mode=m, encoding='utf-8') as cfg:
                    cfg = yaml.load(cfg, Loader=SafeLoader)
                    if type(cfg) == "dict":
                        cfg = dict(cfg)
                    elif type(cfg).__name__ == "list":
                        cfg = list(cfg)

                    config = cfg.get(ref) if ref else cfg
        except Exception as ex:
            print(f'[Chargement du fichier {p}:\n', ex)
        finally:
            return config

    @staticmethod
    def save_cfg(d, f, m="w"):
        """
        Enregistrement d' un fichier
        ========================================

        :param dict(str, list(str)) d: données à enregistrer
        :param str f: nom du fichier
        :param str m: default (write): mode "w|a", optional
        :return:
        """
        profiler.makedirs(profiler.dirparent(f))

        with open(f, m) if 'b' in m else open(f, mode=m, encoding='utf-8') as f_yml:
            yaml.dump(d, stream=f_yml, allow_unicode=True)

        return f


class CFGBases(CFGEngine):
    """
    Cette class permet de gere des fichiers de configuration disponibles dans le repertoire <PROJET_DIR>/cfg
    """
    CFG_DIR = CFGEngine.working_directory('')  # databases parameters
    _logs = profiler.path_build(CFG_DIR, 'log.yml')
    _app = profiler.path_build(CFG_DIR, 'app.yml')
    _categories = profiler.path_build(CFG_DIR, 'categorie.yml')
    _mail = profiler.path_build(CFG_DIR, 'mailing.yml')
    _validator = profiler.path_build(CFG_DIR, 'validators.yml')  # databases parameters
    _normalisator = profiler.path_build(CFG_DIR, 'normalizor.yml')  # databases parameters

    @staticmethod
    def loadingbyref(filename, *args, **kwargs):
        """
        Récupération des parametres de configuration du fichier <filename> section <r>

        :param str filename: Fichier de configuration
        :param str code: référence parametres à récupérer, optionnel
        :param str mode: bytes par defaut
        :return: configuration | None
        """
        filepath = profiler.path_build(CFGBases.CFG_DIR, f'{filename}.yml')
        return CFGEngine.loading(filepath, *args, **kwargs)

    @staticmethod
    def logs_cfg():
        """ Configuration des logs

        :Exemple:
            >>> import import logging.config as log_config
            >>> import logging
            >>> log_config.dictConfig(CFGBases.logs_cfg())
            >>> tracker = logging.getLogger('PROD|TEST')
            >>> tracker.info("Exemple dun message d'information")

        """

        return CFGBases.loading(CFGBases._logs)

    @staticmethod
    def app_cfg(code=None):
        """ Parametres application

        :param str code: clé a retourner (filtre)
        :return: Configuration
        """
        return CFGBases.loading(CFGBases._app, code)

    @staticmethod
    def validator():
        """ Parametres de validation de formulaire
        """
        return CFGBases.loading(CFGBases._validator)

    @staticmethod
    def normalizor():
        """ Parametres de normalisation de formulaire
        :return: parametres de normaisation
        :rtype: dict
        """
        return CFGBases.loading(CFGBases._normalisator)

    @staticmethod
    def mailing_lib(code):
        """ Mail préparé

        :param str code: référence du mail à envoyer
        :return: mail

        """
        return CFGBases.loading(CFGBases._mail, code)

    @staticmethod
    def categorie_lib(code=None):
        """ Liste de definition

        :param str code: référence du de la liste
        :return: liste(s) de categories
        :rtype: dict
        """
        return CFGBases.loading(CFGBases._categories, code)


_all_ = ['CFGBases']



# -*- coding: utf-8 -*-

"""
Module de fonctions basiques
=============================

Liste de fonctions utiles

pathfile : dreamtools/tools

Constantes globales
--------------------
.. note::

    * RGX_ACCENTS = 'àâäãéèêëîïìôöòõùüûÿñç'
    * RGX_EMAIL = Expression reguliere email
    * RGX_PUNCT = Caractere speciaux autorisé pour mot de passe
    * RGX_PWD = Expression régulière pour un mot de passe de 8 à 12 avec un car.Special/une Majuscule/Une minuscule
    * RGX_PHONE = Expression réguliere remative à un numéro de téléphon
    * RGX_URL = expression reguliere pour URL
    * *PROJECT_DIR*  : Repertoire du projet
    * *APP_NAME* : Nom de l'application
    * *APP_DIR* : PROJECT_DIR/APP_NAME
    * *TMP_DIR* : PROJECT_DIR/tmp

.. warning::
    Il faut configurer l'application afin d'avoir accès au variable PROJECT_DIR, APP_NAME, TMP_DIR

    >>> from dreamtools import config
    >>> from dreamtools import tools
    >>> config.CConfig('monapp')
    >>> print (tools.APP_DIR)
    '../PROJECT/mon_app'

"""

import ast
import re
import sys
from random import choice, randint
from string import punctuation, ascii_letters, digits

RGX_ACCENTS = 'àâäãéèêëîïìôöòõùüûÿñç'
RGX_EMAIL = r'^[a-z0-9_.+-]+@[a-z0-9-]+\.[a-z0-9-.]+$'
RGX_PUNCT = '#!?$%&_@*+-'
RGX_PWD = fr'.*(?=.{{8,12}})(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[{RGX_PUNCT}]).*'
RGX_PHONE = r'^0[1-9]\d{8}$'
RGX_URL = r'https?:\/\/(www\.)?[-a-z0-9@:%._\+~#=]{1,256}\.[a-z0-9()]{1,6}\b([-a-z0-9()@:%_\+.~#?&//=]*)'

APP_NAME = ''
PROJECT_DIR = ''
APP_DIR = ''
TMP_DIR = ''


def print_err(*args, **kwargs):
    """Ecriture sur le flux erreur de la console

    :param args: arguments 1
    :param kwargs: arguemnts2
    """
    print(*args, file=sys.stderr, **kwargs)


def string_me(v):
    """ Convertion d'une valeur en chaine

    :param v: valeur à convertir
    :rtype: str, None en cas d'erreur

    """

    try:
        return str(v)
    except:
        return None


def clean_space(ch):
    """ Nettoyage des espaces "superflus"

    * Espaces à gouche et à droite supprimés
    * Répétition d'espace réduit

    :Exemple:
        >>> chaine = 'Se  réveiller au matin        de sa destiné    ! !           '
        >>> clean_space (chaine)
        'Se réveiller au matin  de sa destiné ! !'

    """
    s = string_me(ch)
    return re.sub(r'\s{2,}', ' ', s.strip())


def clean_allspace(ch, very_all=True):
    """Nettoyage de tous les espace et carateres vides

    :param str ch: Chaine à nettoyer
    :param bool very_all: caractère vide aussi, True (False = Espaces uniquement)

    :Exemple:
        >>> chaine = 'Se  réveiller au matin        de sa destiné !'
        >>> clean_allspace (chaine)
        'Seréveilleraumatindesadestiné!'

    """
    c = r'\s' if very_all else '[ ]'
    s = string_me(ch)

    return re.sub(c, '', s.strip())


def clean_coma(ch, w_punk=False):
    """ Supprime les accents/caractères spéciaux du texte source en respectant la casse

    :param ch: Chaine de caractere à "nettoyer"
    :param w_punk: indique si la punctuation est à nettoyer ou pas (suppression)

    :Exemple:
        >>> s = 'Se  réveiller au matin    de sa destiné !!'
        >>> clean_coma (s)
        'Se seveiller au matin (ou pas) de sa destine !!''
        >>> clean_coma (s, True)
        'Se reveiller au matin ou pas de sa destine'

    """

    if w_punk:
        # Nettoyage caractere spéciaux (espace...)
        o_rules = str.maketrans(RGX_ACCENTS, 'aaaaeeeeiiioooouuuync', punctuation)
    else:
        o_rules = str.maketrans(RGX_ACCENTS, 'aaaaeeeeiiioooouuuync')

    return clean_space(ch).translate(o_rules).swapcase().translate(o_rules).swapcase()


def clean_master(ch):
    """ Supprime les accents, caractères spéciaux et espace du texte source

    :param str ch: Chaine de caractere à "nettoyer"
    :return str: chaine sans accents:car. spéciaux ni espace en minuscule

    :Exemple:
        >>> s = 'Se  réveiller au matin  (ou pas) de sa destiné !'
        >>> clean_master (s)
        'sereveilleraumatinoupasdesadestine

    """
    return clean_allspace(clean_coma(ch, True)).lower()


def inttohex(v):
    """ Conversion d'une valeur en hexadécimal

    :param int v: nombre à convertir
    :returns: valeur en hexadécimal
    :rtype: str

    """

    return hex(int(v))


def addhex(h, v):
    """Additionne une valeur hexadécimal

    :param str h: valeur hexadécimal
    :param int v: valeur entière à ajouter
    :return: valeur additionné en hexedécimal

    :Example:
        >>> hx = '0x129'
        >>> addhex(hx, 2)
        0x12b

    """

    v += int(h, 16)
    return hex(v)


def plain_hex(hx, s=3):
    """ Complète un chiffre hexadecimal en préfixant une valeur de zéro

    :param str hx: valeur hexadécimal
    :param int s: longeur chaine attendu
    :rtype: str:

    :Examples:
        >>> hx = '0x129'
        >>> plain_hex(hx, 5)
        0x00129

    """
    return hx[:2] + plain_zero(hx[2:], s)


def plain_zero(v, s):
    """Complete une valeur chaine de zéro

    :param v: valeur à completer
    :param s: taille chaine attendu préfixé de zerom

    :Exemple:
        >>> d = 5
        >>> plain_zero(d,3)
        '005'

    """

    s = '{:0>' + str(s) + '}'
    return s.format(v)


def check_password(s):
    """ Vérifie que la syntaxe d'une chaine répond au critère d'un mot de passe

    :Conditions:
        * Une majuscule
        * Une minuscule
        * Un chiffre
        * Un carectère spécial (@#!?$&-_ autorisé )

    :param str s: chaine à vérifier
    :return bool: True si la chaine est valide
    """
    r = re.compile(RGX_PWD)
    return r.match(s)


def comphex(hx_a, hx_b):
    """Compare deux valeurs  hexadécimales

    :param str hx_a:
    :param str hx_b:

    :return int:
        * 0 : hx_a == hx_b
        * 1 : hx_a > hx_b
        * -1 : hx_a < hx_b
    """
    v = int(hx_a, 16) - int(hx_b, 16)

    if v == 0:
        return 0
    else:
        return -1 if v < 0 else 1


def pwd_maker(i_size=8):
    """ Génération d'un password respectant les regles de password

    :Conditions:
        * Une majuscule
        * Une minuscule
        * Un chiffre
        * Un carectère spécial (@#!?$&-_ autorisé )

    :param int i_size: Nombre de caracteres de la chaine
    :return: Mot de passe
    """

    t = list(ascii_letters + digits + RGX_PUNCT)

    while True:
        s_chaine = ''
        for i in range(0, i_size):
            s = choice(t)
            s_chaine += s
            t.remove(s)
        if check_password(s_chaine):
            break

    return s_chaine


def code_maker(i_size=4):
    """Génération d'une chaine aléatoire composé de lettre et de chiffres

    :param int i_size: taille du code
    :rtype str:
    """
    ll = list(ascii_letters + digits)
    s = ''

    for i in range(0, i_size):
        s += choice(ll)

    return s


def aleatoire(end, s=1):
    """ Génération d'un nombre aléatoire entre [1-end] => end caractère

    :param int end: valeur maximal (paut indiquer la taille si s=1)
    :param s: valeur de départ, default to 1
    :return: Un chiffre aléatoire

    :Exemple:
        >>> aleatoire (5)
        1 : Renvoie un chiffre entre 1 et 5
        >>> aleatoire (5,3)
        1 : Renvoie un chiffre entre 3 et 5
        >>> 4

    """

    return randint(s, end)


def add_list(v, ll):
    """ Ajout d'un item dans une liste avec gestion des doublons

    :param str v: valeur à ajouter
    :param list ll: liste

    """
    if v not in ll:
        ll.append(v)


def dictlist(k, v, d):
    """ Ajout d'un valeur dans une liste d'un dictionnaire

    :param str k: clé dictionnaire
    :param v: valeur à ajouter
    :param dict[str, list[]] d: dictionnaire

    :Exemple:
        >>> dictionnaire= {}
        >>> dictlist('printemps', 'mar', dictionnaire)
        dictionnaire{'printemps', ['mars']}
        >>> dictlist('printemps', 'avril', dictionnaire)
        dictionnaire{'printemps', ['mars', ''avril']}
        >>> dictlist('printemps', 'mars', dictionnaire)
        dictionnaire{'printemps', ['mars', ''avril']}

    """
    if k is None or v is None: return

    if k not in d: d[k] = []

    add_list(v, d.get(k))


def str_dic(chaine):
    """Convertion d'une chaine en dictionnaire

    :param str chaine:
    :rtype: dic

    :Exemple:
        >>> s_dic = "{'key':value}"
        >>> str_dic(s_dic)
        {'key': 'value'}

    """

    return ast.literal_eval(chaine)


def pop_dic(l_ids, dic):
    """ Suppression d'une liste d'éléments d'un dictionnaire

    :param list[str] l_ids : liste de clé à supprimer
    :param dict[str:object] dic: dictionaire à nettoyer

    """
    if dic:
        for s in l_ids:
            if s in dic: del dic[s]

def dicFindKey (value, dic):
    """Recherche une clé d'un dictionnaire à partir de sa valeur"""
    for k, v in dic.items():
        if v == value:
            return k
    else:
        return None


def htmltexted (texte):
    import bs4

    try:
        bs = bs4.BeautifulSoup(texte, features="html.parser")
        texte = bs4.UnicodeDammit(bs.get_text()).unicode_markup
    except:
        pass
    finally:
        return tools.clean_space(texte)


#!/usr/bin/python3
# -*- coding: utf-8 -*-
# krypt.py

"""
Module de cryptage
=========================================
pathfile : dreamtools/tools

Pré-Requis
----------------------
Une clé privé et un grain de sel doivent être défini dans le fichier "de parapetre d'application"
**PROJECT_DIR/cfg/.app.yml**
* SECRET_KEY
* SALT_EXT

*Méthode* : sha-512`

Class CKrypting
----------------
"""

import crypt
import os


class CKrypting:
    """
    Gestion kryptage des données
    """
    PREFIX = "$5$"
    SALT = os.getenv('SECRET_KEY')
    SALT_EXT = os.getenv('EXTERN_SECRET_KEY')

    @staticmethod
    def __encrypt(ch, prefix):
        if isinstance(ch, bytes):
            orig = ch
            try:
                ch = ch.decode("utf-8")
            except UnicodeDecodeError:
                return None
            assert ch.encode("utf-8") == orig, "utf-8 spec says this can't happen!"

        return crypt.crypt(ch, prefix)

    @staticmethod
    def encrypt(ch):
        """Crypte une chaine selon la clé privé et le grain de sel paramétrés

        :param str ch: Chaine à crypter
        :return str: Chaine cryptée

        """

        return CKrypting.__encrypt(ch, CKrypting.PREFIX + CKrypting.SALT)

    @staticmethod
    def extern_encrypt(ch):
        """
        Cryptage chaine pour utilisation extrene (suppression prefix)

        :param str ch:
        :return: mot de passe crypté - grain de sel
        """
        """This is used in place of `mkpasswd --sha-512`"""
        s = CKrypting.__encrypt(ch, CKrypting.PREFIX + CKrypting.SALT_EXT)
        i = len(CKrypting.SALT_EXT)

        return s[i:]

    @staticmethod
    def compare(enc, s):
        """Comparaison d'un chaine en clair à un mot de passe crypté

        :param str enc: chaine cryptée
        :param str s: chaine en claire
        :rtype: bool

        """
        return crypt.crypt(s, enc) == enc



import unittest

from dreamtools import tools
from dreamtools.config import CConfig
from dreamtools.logmng import CTracker

CConfig('dreamtools')


class MyTestCase(unittest.TestCase):
    @staticmethod
    def testinit():
        print('Configuration')
        print(tools.string_me(55))

    @staticmethod
    def testlog():
        CTracker.info_tracking('test message info', 'je test')


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/python3
# -*- coding: utf-8 -*-
# project/dreamtools/imagine.py

import os
from io import BytesIO

from PIL import Image
from PIL.ExifTags import TAGS
from PIL.TiffImagePlugin import ImageFileDirectory_v2

from . import (tools, profiler)
from .logmng import CTracker

TYPE_IMG_JPEG = 'JPEG'
TYPE_IMG_PNG = 'PNG'
TYPE_IMG_GIF = 'GIF'

"""
Class CImagine
=============================

Class permettant le traitement d'une image png convertit en jpg avec prise en charge de la transparence (fond blanc)


pathfile : dreamtools/pyimaging

"""


class CImagine(object):

    def __init__(self, src, dest, with_ext=False):
        """
        Preparation image  pour traitement
        ==================================
        Les images sont convertit au format JPEG

        :parametres:
        :param str src: pathfile image d'origine
        :param src dest: path destionantion image
        :param bool with_ext: Avec ou sans extension, False par défaut
        """

        self.img = Image.open(src)
        self.exif = None

        if self.img.mode != 'RGB':
            self.img = self.img.convert('RGB')

        self._size = self.img.size
        self.format = self.img.format

        if self.format != TYPE_IMG_JPEG:
            try:
                self.img = self.white_background
            except Exception as ex:  # pas de fond transparent
                tools.print_err("imagine.init Ajout fond blanc", ex)

        self.file = profiler.file_ext_less(dest)     # on s'assure de retirer l'extension

        if with_ext:
            self.file += '.jpg'

        self.thumb = (200, 200)

        self.min_size = 250
        self.max_size = 500

    def __enter__(self):
        return self

    @property
    def _size(self):
        return self.w, self.h

    @_size.setter
    def _size(self, s):
        self.w, self.h = s

    @property
    def white_background(self):
        """
        Ajout d'un fond transparent
        :return:
        """
        bg = Image.new("RGB", self._size, (255, 255, 255))
        bg.paste(self.img, self.img)

        return bg

    def resize(self, s=None, mn=None, mx=None):
        """ Redimensionnement de l'image au format jpg

        :param tuple(int, int) s: si indiqué, (w,h) de redimensionnement
        :param int mn,optional: taille maximum (carré), default 250
        :param int mx,optional: taille maximum de l'image,  default 250
        :return:
        """

        if s is None:
            min_size = mn or self.min_size
            max_size = mx or self.max_size

            h = self.h
            w = self.w

            if self.w >= self.h:
                if not (min_size <= self.w <= max_size):
                    s_ref = min_size if self.w < min_size else max_size
                    h = self.h * s_ref / self.w
                    w = s_ref
            elif not (min_size <= self.h <= max_size):
                s_ref = min_size if self.h < min_size else max_size
                w = self.w * s_ref / self.h
                h = s_ref

            s = (int(w), int(h))

        img = self.img.resize(s)
        img.save(self.file, TYPE_IMG_JPEG)

    def thumbed(self, s=None):
        """ Thumb Image

        :param tuple[int, int] s: taille image, defaul (200, 200à

        """

        img = self.img.convert('L')
        img.thumbnail(s or self.thumb)

        if self.exif:
            img.save(self.file + ".thumb", TYPE_IMG_JPEG, exif=self.exif)
        else:
            img.save(self.file + ".thumb", TYPE_IMG_JPEG)

    def __exit__(self, exc_type, exc_value, traceback):
        self.img.close()

    def protected(self, artist, description):
        """Ajoute un nom d'artist et le copyright d'une image"""

        _TAGS_r = dict(((v, k) for k, v in TAGS.items()))

        # Image File Directory
        ifd = ImageFileDirectory_v2()
        ifd[_TAGS_r["Artist"]] = artist
        ifd[_TAGS_r["Copyright"]] = 'Tous droits reserves'
        ifd[_TAGS_r["Description"]] = description

        out = BytesIO()
        ifd.save(out)

        self.exif = b"Exif\x00\x00" + out.getvalue()
        self.img.save(self.file, TYPE_IMG_JPEG, exif=self.exif)

    @classmethod
    def treat_dir(cls, s):
        """
        Redimensionne toutes les images contenu dans un répertoire donné + thumb
        :param s:
        """
        for f in os.listdir(s):
            f_path = os.path.join(s, f)
            if os.path.isfile(f_path):
                cls.treat_img(f_path, f_path)

    @classmethod
    def treat_img(cls, s, f):
        """
        Enregistre une image, la reformat + thumb
        :param s:
        :param f:
        """
        o = cls(s, f)
        o.resize()
        o.thumbed()

    @classmethod
    def imgrecorder(cls, fs, fp):
        """
        Enregistremetn d'une image uploaded (byte)

        :param file fs: filestream from flask request
        :param str fp: filepath d'enregistrement
        :return:
        """
        import uuid

        tmp = uuid.uuid1().int >> 64
        tmp = profiler.path_build(tools.TMP_DIR, str(tmp))

        profiler.makedirs(tools.TMP_DIR)

        try:
            CTracker.flag(f'[tools.imgrecorder] Image temp recording : {tmp}')
            fs.save(tmp)

            CTracker.flag(f'[tools.imgrecorder] Image treatment: {tmp} | {fp}')
            o = cls(tmp, fp)
            CTracker.flag(f'[tools.imgrecorder] Image resizing: {fp}')
            o.resize()
            CTracker.flag('[tools.imgrecorder] Image thumbing')
            o.thumbed()

            CTracker.flag('[tools.imgrecorder] Temp Cleaning')
            profiler.makedirs(tmp)
            return True
        except Exception as e:
            CTracker.exception_tracking(e, '[tools.imgrecorder] Aïe')
            profiler.remove_file(tmp)
            return False



# -*- coding: utf-8 -*-
# mailbot.py

"""
Module de Gestion de mail préparés

pathfile : dreamtools/mailbot.py

Pré-Requis
-------------
.. warning::

    Indiquer les parametres smtp dans le fichiers de configuration <PROJECT_NAME>/cfg/.app.yml

.. code-block:: YAML

    SMTP_HOST: smtp-host_adresse
    SMTP_PORT: port_smtp
    SMTP_AUTHMAIL: mail_authen
    SMTP_AUTHPWD: password_auth
    SMTP_USERNAME : name_sender <email>

.. warning::

    Les mails sont à définir dans le ficchier <PROJECT_NAME>/cfg/mailing.yml au format suivant

.. code-block:: YAML

     footer:
      html: <Pied de mail unique pour tous les mails (signature, rgpd...)>
      text: <Pied de mail unique pour tous les mails (signature, rgpd...)>
     code_mail:
      html: <ici mail au format HTML>
      text : <Le mail au format texte>
      objt : <Objet du mail>


"""
import os
import smtplib
import ssl
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from .cfgmng import CFGBases
from .logmng import CTracker


class CSMTP(object):
    SMTP_HOST = os.getenv('SMTP_HOST')
    SMTP_PORT = os.getenv('SMTP_PORT')
    SMTP_AUTHMAIL = os.getenv('SMTP_AUTHMAIL')
    SMTP_AUTHPWD = os.getenv('SMTP_AUTHPWD')
    SMTP_USERNAME = f"{os.getenv('SMTP_USERNAME')} <{os.getenv('SMTP_AUTHMAIL')}>"


class CMailer(CSMTP):
    footers = CFGBases.mailing_lib('footer')

    @classmethod
    def __send_mail(cls, subject, receivers, d_msg, to_receiver=None):
        """ Envoie du mail

        :param subject: Sujet du mail
        :param receivers: email destinataire
        :param d_msg: Message
        :param to_receiver: Nom destinataire
        :return:
        """

        CTracker.flag("[dreamtools.mailbot] SEND_MAIL : Parametrage smtp")
        context = ssl.create_default_context()

        CTracker.flag("[dreamtools.mailbot] SEND_MAIL:Parametrage message MIME")
        message = MIMEMultipart("alternative")
        message["Subject"] = subject
        message["From"] = cls.SMTP_USERNAME
        message["To"] = to_receiver or receivers

        CTracker.flag("[dreamtools.mailbot] SEND_MAIL:Parametrage contenu mail")
        content = d_msg.get('text') + cls.footers['text']
        content = MIMEText(content)
        message.attach(content)

        if d_msg.get('html'):
            content = d_msg.get('html') + cls.footers['html']
            content = MIMEText(content, "html")
            message.attach(content)

        CTracker.flag("[dreamtools.mailbot] SEND_MAIL:Coonnexion SMTP")
        with smtplib.SMTP_SSL(cls.SMTP_HOST, cls.SMTP_PORT, context=context) as server:
            CTracker.flag("[dreamtools.mailbot] SEND_MAIL:Authentification")
            server.login(cls.SMTP_AUTHMAIL, cls.SMTP_AUTHPWD)
            CTracker.flag("[dreamtools.mailbot] SEND_MAIL: Sending")
            server.sendmail(cls.SMTP_AUTHMAIL, receivers, message.as_string())

            return True

    @classmethod
    def presend(cls, email, code, name='', **data_field):
        """ Preparation pour envoi d'un message mail 
        
        :param str email: email destinataire
        :param str code: réfénce du mail à chargé
        :param str name: nom du destinataire
        :param dict data_field: liste de données relatif à des champs définis dans le mails

        
        """
        CTracker.flag('[dreamtools.mailbot] PRESEND:Loading template {}'.format(code))
        mail = CFGBases.mailing_lib(code)

        CTracker.flag('[dreamtools.mailbot] PRESEND: Preparation')

        part1 = mail['text'].format(**data_field)
        part2 = mail['html'].format(**data_field)
        to_receiver = r'{} <{mail}>'.format(name, mail=email)

        CTracker.flag('[dreamtools.mailbot] PRESEND: Envoi ({}) -> {}'.format(code, email))
        send = CTracker.fnCTracker(cls.__send_mail, 'Envoi ({}) -> {}'.format(code, email), mail.get('object'),
                                 email, {'text': part1, 'html': part2}, to_receiver)

        return send.ok


__all__ = ['CMailer']
